New option: --mountpoint.
[dss.git] / snap.h
diff --git a/snap.h b/snap.h
index 1e9fc2b87423ae520ef6f3b6a2b2d7e3d0b52e89..626ecad003a52a1a465ea6f7f2e11f960a97df3c 100644 (file)
--- a/snap.h
+++ b/snap.h
@@ -1,36 +1,59 @@
-enum {
-       /** We are ready to take the next snapshot. */
-       SCS_READY,
-       /** The pre-creation hook has been started. */
-       SCS_PRE_HOOK_RUNNING,
-       /** The pre-creation hook exited successfully. */
-       SCS_PRE_HOOK_SUCCESS,
-       /** The rsync process is running. */
-       SCS_RSYNC_RUNNING,
-       /** The rsync process exited successfully. */
-       SCS_RSYNC_SUCCESS,
-       /** The post-create hook has been started- */
-       SCS_POST_HOOK_RUNNING,
-};
-
 /*
- * complete, not being deleted: 1204565370-1204565371.Sun_Mar_02_2008_14_33-Sun_Mar_02_2008_14_43
- * complete, being deleted: 1204565370-1204565371.being_deleted
- * incomplete, not being deleted: 1204565370-incomplete
- * incomplete, being deleted: 1204565370-incomplete.being_deleted
+ * Copyright (C) 2008-2010 Andre Noll <maan@tuebingen.mpg.de>
+ *
+ * Licensed under the GPL v2. For licencing details see COPYING.
+ */
+
+
+/** The possible states for snapshot creation/removal. */
+#define HOOK_STATUS_ARRAY \
+       HSA_ITEM(HS_READY, "ready for action"), \
+       HSA_ITEM(HS_PRE_RUNNING, "pre-hook running"), \
+       HSA_ITEM(HS_PRE_SUCCESS, "pre-hook completed successfully"), \
+       HSA_ITEM(HS_RUNNING, "in progress"), \
+       HSA_ITEM(HS_SUCCESS, "process terminated successfully"), \
+       HSA_ITEM(HS_NEEDS_RESTART, "restart needed"), \
+       HSA_ITEM(HS_POST_RUNNING, "post-hook running")
+
+
+#define HSA_ITEM(x, y) x
+enum hook_status {HOOK_STATUS_ARRAY};
+#undef HSA_ITEM
+#define HSA_ITEM(x, y) y
+
+
+/**
+ * The status of a snapshot.
+ *
+ * The snapshot directories come in four different flavours, depending
+ * on how the two status flags are set. Examples:
+ *
+ * Complete, not being deleted: 1204565370-1204565371.Sun_Mar_02_2008_14_33-Sun_Mar_02_2008_14_43.
+ * Complete, being deleted: 1204565370-1204565371.being_deleted.
+ * Incomplete, not being deleted: 1204565370-incomplete.
+ * incomplete, being deleted: 1204565370-incomplete.being_deleted.
  */
 enum snapshot_status_flags {
        /** The rsync process terminated successfully. */
        SS_COMPLETE = 1,
        /** The rm process is running to remove this snapshot. */
-       SS_BEING_DELETED = 2,
+       SS_BEING_DELETED = 2
 };
 
+/** Describes one snapshot. */
 struct snapshot {
+       /** The name of the directory, relative to the destination dir. */
        char *name;
+       /** Seconds after the epoch when this snapshot was created. */
        int64_t creation_time;
+       /**
+        * Seconds after the epoch when creation of this snapshot completed.
+        * Only meaningful if the SS_COMPLETE bit is set.
+        */
        int64_t completion_time;
+       /** See \ref snapshot_status_flags. */
        enum snapshot_status_flags flags;
+       /** The interval this snapshot belongs to. */
        unsigned interval;
 };
 
@@ -48,9 +71,11 @@ struct snapshot_list {
        unsigned *interval_count;
 };
 
+/** Iterate over all snapshots in a snapshot list. */
 #define FOR_EACH_SNAPSHOT(s, i, sl) \
        for ((i) = 0; (i) < (sl)->num_snapshots && ((s) = (sl)->snapshots[(i)]); (i)++)
 
+/** Iterate backwards over all snapshots in a snapshot list. */
 #define FOR_EACH_SNAPSHOT_REVERSE(s, i, sl) \
        for ((i) = (sl)->num_snapshots; (i) > 0 && ((s) = (sl)->snapshots[(i - 1)]); (i)--)
 
@@ -63,10 +88,14 @@ __malloc char *incomplete_name(int64_t start);
 __malloc char *being_deleted_name(struct snapshot *s);
 int complete_name(int64_t start, int64_t end, char **result);
 __malloc char *name_of_newest_complete_snapshot(struct snapshot_list *sl);
+int num_complete_snapshots(struct snapshot_list *sl);
 
-static inline struct snapshot *get_oldest_snapshot(struct snapshot_list *sl)
+/**
+ * Get the newest snapshot in a snapshot list.
+ */
+_static_inline_ struct snapshot *get_newest_snapshot(struct snapshot_list *sl)
 {
        if (!sl->num_snapshots)
                return NULL;
-       return sl->snapshots[0];
+       return sl->snapshots[sl->num_snapshots - 1];
 }